jest.mock('app/login/login.service');

import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { of } from 'rxjs';
import { NgxWebstorageModule } from 'ngx-webstorage';
import { TranslateModule } from '@ngx-translate/core';

import { ProfileInfo } from 'app/layouts/profiles/profile-info.model';
import { Account } from 'app/core/auth/account.model';
import { AccountService } from 'app/core/auth/account.service';
import { ProfileService } from 'app/layouts/profiles/profile.service';
import { LoginService } from 'app/login/login.service';

import { ImprovementsRequestedMenuBadgeComponent, ReviewMenuBadgeComponent, ReviewRequestedMenuBadgeComponent } from './navbar.component';
import { ReactiveFormsModule } from '@angular/forms';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { ReviewManagementService } from 'app/admin/review-management/review-management.service';

describe('ReviewMenuBadgeComponent Component', () => {
  let comp: ReviewMenuBadgeComponent;
  let fixture: ComponentFixture<ReviewMenuBadgeComponent>;
  let accountService: AccountService;
  let reviewManagementService: ReviewManagementService;
  let httpMock: HttpTestingController;
  let applicationConfigService: ApplicationConfigService;
  const stats = {
    reviewInProgress: 0,
    reviewImproved: 0,
    reviewCompleted: 0,
    unknownStats: 0,
    reviewsRequestedByUser: 2,
    reviewInProgressByUser: 0,
    reviewImprovementRequestedByUser: 2,
    reviewCompletedByUser: 0,
    unknownStatsByUser: 0,
    badgesRewarded: 1,
  };

  beforeEach(
    waitForAsync(() => {
      TestBed.configureTestingModule({
        imports: [
          HttpClientTestingModule,
          RouterTestingModule.withRoutes([]),
          ReactiveFormsModule,
          TranslateModule.forRoot(),
          NgxWebstorageModule.forRoot(),
        ],
        declarations: [ReviewMenuBadgeComponent],
        providers: [LoginService, ReviewManagementService],
      }).compileComponents();
    })
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(ReviewMenuBadgeComponent);
    comp = fixture.componentInstance;
    accountService = TestBed.inject(AccountService);
    applicationConfigService = TestBed.inject(ApplicationConfigService);
    reviewManagementService = TestBed.inject(ReviewManagementService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  it('Should call reviewManagementService.getReviewStatistics on init', () => {
    // WHEN
    comp.ngOnInit();
    // WHEN

    const testRequest = httpMock.expectOne({
      method: 'POST',
      url: applicationConfigService.getEndpointFor('api/review/countAllByUser'),
    });
    testRequest.flush(stats);

    expect(comp.getReviewActionCount()).toEqual(stats.reviewInProgress + stats.reviewImproved);
    //expect(comp.getReviewActionLabel()).toEqual("2 reviews in progress or improvement requested.");
    expect(comp.getReviewActionLabel()).toEqual(''); // still a race condition
  });
});

describe('ReviewRequestedMenuBadge Component', () => {
  let comp: ReviewRequestedMenuBadgeComponent;
  let fixture: ComponentFixture<ReviewRequestedMenuBadgeComponent>;
  let accountService: AccountService;
  let reviewManagementService: ReviewManagementService;
  let httpMock: HttpTestingController;
  let applicationConfigService: ApplicationConfigService;
  const stats = {
    reviewInProgress: 1,
    reviewImproved: 2,
    reviewCompleted: 3,
    unknownStats: 4,
    reviewsRequestedByUser: 5,
    reviewInProgressByUser: 6,
    reviewImprovementRequestedByUser: 7,
    reviewCompletedByUser: 8,
    unknownStatsByUser: 9,
    badgesRewarded: 10,
  };

  beforeEach(
    waitForAsync(() => {
      TestBed.configureTestingModule({
        imports: [
          HttpClientTestingModule,
          RouterTestingModule.withRoutes([]),
          ReactiveFormsModule,
          TranslateModule.forRoot(),
          NgxWebstorageModule.forRoot(),
        ],
        declarations: [ReviewMenuBadgeComponent],
        providers: [LoginService, ReviewManagementService],
      }).compileComponents();
    })
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(ReviewRequestedMenuBadgeComponent);
    comp = fixture.componentInstance;
    accountService = TestBed.inject(AccountService);
    applicationConfigService = TestBed.inject(ApplicationConfigService);
    reviewManagementService = TestBed.inject(ReviewManagementService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  it('Should call reviewManagementService.getReviewStatistics on init', () => {
    // WHEN
    comp.ngOnInit();
    // WHEN

    const testRequest = httpMock.expectOne({
      method: 'POST',
      url: applicationConfigService.getEndpointFor('api/review/countAllByUser'),
    });
    testRequest.flush(stats);

    expect(comp.getReviewRequestedCount()).toEqual(stats.reviewsRequestedByUser);
    expect(comp.getReviewRequestedLabel()).toEqual('5 reviews request, but not yet assigned.');
  });
});

describe('ImprovementsRequestedMenuBadge Component', () => {
  let comp: ImprovementsRequestedMenuBadgeComponent;
  let fixture: ComponentFixture<ImprovementsRequestedMenuBadgeComponent>;
  let accountService: AccountService;
  let reviewManagementService: ReviewManagementService;
  let httpMock: HttpTestingController;
  let applicationConfigService: ApplicationConfigService;
  const stats = {
    reviewInProgress: 1,
    reviewImproved: 2,
    reviewCompleted: 3,
    unknownStats: 4,
    reviewsRequestedByUser: 5,
    reviewInProgressByUser: 6,
    reviewImprovementRequestedByUser: 7,
    reviewCompletedByUser: 8,
    unknownStatsByUser: 9,
    badgesRewarded: 10,
  };

  beforeEach(
    waitForAsync(() => {
      TestBed.configureTestingModule({
        imports: [
          HttpClientTestingModule,
          RouterTestingModule.withRoutes([]),
          ReactiveFormsModule,
          TranslateModule.forRoot(),
          NgxWebstorageModule.forRoot(),
        ],
        declarations: [ReviewMenuBadgeComponent],
        providers: [LoginService, ReviewManagementService],
      }).compileComponents();
    })
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(ImprovementsRequestedMenuBadgeComponent);
    comp = fixture.componentInstance;
    accountService = TestBed.inject(AccountService);
    applicationConfigService = TestBed.inject(ApplicationConfigService);
    reviewManagementService = TestBed.inject(ReviewManagementService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  it('Should call reviewManagementService.getReviewStatistics on init', () => {
    // WHEN
    comp.ngOnInit();
    // WHEN

    const testRequest = httpMock.expectOne({
      method: 'POST',
      url: applicationConfigService.getEndpointFor('api/review/countAllByUser'),
    });
    testRequest.flush(stats);

    expect(comp.getReviewRequestedCount()).toEqual(7);
    expect(comp.getReviewRquestedLabel()).toEqual('7 review(s) resulted in an improvements request');
  });
});
